package org.cytoscape.ding.internal.util;

/*
 * #%L
 * Cytoscape Ding View/Presentation Impl (ding-presentation-impl)
 * $Id:$
 * $HeadURL:$
 * %%
 * Copyright (C) 2006 - 2013 The Cytoscape Consortium
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation, either version 2.1 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 * 
 * You should have received a copy of the GNU General Lesser Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-2.1.html>.
 * #L%
 */

import java.awt.Color;
import java.util.HashMap;
import java.util.Map;

// TODO delete?
public enum ColorKeyword {
	SNOW ("snow", 0xfffafa),
	GHOSTWHITE ("ghostwhite", 0xf8f8ff),
	WHITESMOKE ("whitesmoke", 0xf5f5f5),
	GAINSBORO ("gainsboro", 0xdcdcdc),
	FLORALWHITE ("floralwhite", 0xfffaf0),
	OLDLACE ("oldlace", 0xfdf5e6),
	LINEN ("linen", 0xfaf0e6),
	ANTIQUEWHITE ("antiquewhite", 0xfaebd7),
	PAPAYAWHIP ("papayawhip", 0xffefd5),
	BLANCHEDALMOND ("blanchedalmond", 0xffebcd),
	BISQUE ("bisque", 0xffe4c4),
	PEACHPUFF ("peachpuff", 0xffdab9),
	NAVAJOWHITE ("navajowhite", 0xffdead),
	MOCCASIN ("moccasin", 0xffe4b5),
	CORNSILK ("cornsilk", 0xfff8dc),
	IVORY ("ivory", 0xfffff0),
	LEMONCHIFFON ("lemonchiffon", 0xfffacd),
	SEASHELL ("seashell", 0xfff5ee),
	HONEYDEW ("honeydew", 0xf0fff0),
	MINTCREAM ("mintcream", 0xf5fffa),
	AZURE ("azure", 0xf0ffff),
	ALICEBLUE ("aliceblue", 0xf0f8ff),
	LAVENDER ("lavender", 0xe6e6fa),
	LAVENDERBLUSH ("lavenderblush", 0xfff0f5),
	MISTYROSE ("mistyrose", 0xffe4e1),
	WHITE ("white", 0xffffff),
	BLACK ("black", 0x000000),
	DARKSLATEGRAY ("darkslategray", 0x2f4f4f),
	DARKSLATEGREY ("darkslategrey", 0x2f4f4f),
	DIMGRAY ("dimgray", 0x696969),
	DIMGREY ("dimgrey", 0x696969),
	SLATEGRAY ("slategray", 0x708090),
	SLATEGREY ("slategrey", 0x708090),
	LIGHTSLATEGRAY ("lightslategray", 0x778899),
	LIGHTSLATEGREY ("lightslategrey", 0x778899),
	GRAY ("gray", 0xbebebe),
	GREY ("grey", 0xbebebe),
	LIGHTGREY ("lightgrey", 0xd3d3d3),
	LIGHTGRAY ("lightgray", 0xd3d3d3),
	MIDNIGHTBLUE ("midnightblue", 0x191970),
	NAVY ("navy", 0x000080),
	NAVYBLUE ("navyblue", 0x000080),
	CORNFLOWERBLUE ("cornflowerblue", 0x6495ed),
	DARKSLATEBLUE ("darkslateblue", 0x483d8b),
	SLATEBLUE ("slateblue", 0x6a5acd),
	MEDIUMSLATEBLUE ("mediumslateblue", 0x7b68ee),
	LIGHTSLATEBLUE ("lightslateblue", 0x8470ff),
	MEDIUMBLUE ("mediumblue", 0x0000cd),
	ROYALBLUE ("royalblue", 0x4169e1),
	BLUE ("blue", 0x0000ff),
	DODGERBLUE ("dodgerblue", 0x1e90ff),
	DEEPSKYBLUE ("deepskyblue", 0x00bfff),
	SKYBLUE ("skyblue", 0x87ceeb),
	LIGHTSKYBLUE ("lightskyblue", 0x87cefa),
	STEELBLUE ("steelblue", 0x4682b4),
	LIGHTSTEELBLUE ("lightsteelblue", 0xb0c4de),
	LIGHTBLUE ("lightblue", 0xadd8e6),
	POWDERBLUE ("powderblue", 0xb0e0e6),
	PALETURQUOISE ("paleturquoise", 0xafeeee),
	DARKTURQUOISE ("darkturquoise", 0x00ced1),
	MEDIUMTURQUOISE ("mediumturquoise", 0x48d1cc),
	TURQUOISE ("turquoise", 0x40e0d0),
	CYAN ("cyan", 0x00ffff),
	LIGHTCYAN ("lightcyan", 0xe0ffff),
	CADETBLUE ("cadetblue", 0x5f9ea0),
	MEDIUMAQUAMARINE ("mediumaquamarine", 0x66cdaa),
	AQUAMARINE ("aquamarine", 0x7fffd4),
	DARKGREEN ("darkgreen", 0x006400),
	DARKOLIVEGREEN ("darkolivegreen", 0x556b2f),
	DARKSEAGREEN ("darkseagreen", 0x8fbc8f),
	SEAGREEN ("seagreen", 0x2e8b57),
	MEDIUMSEAGREEN ("mediumseagreen", 0x3cb371),
	LIGHTSEAGREEN ("lightseagreen", 0x20b2aa),
	PALEGREEN ("palegreen", 0x98fb98),
	SPRINGGREEN ("springgreen", 0x00ff7f),
	LAWNGREEN ("lawngreen", 0x7cfc00),
	GREEN ("green", 0x00ff00),
	CHARTREUSE ("chartreuse", 0x7fff00),
	MEDIUMSPRINGGREEN ("mediumspringgreen", 0x00fa9a),
	GREENYELLOW ("greenyellow", 0xadff2f),
	LIMEGREEN ("limegreen", 0x32cd32),
	YELLOWGREEN ("yellowgreen", 0x9acd32),
	FORESTGREEN ("forestgreen", 0x228b22),
	OLIVEDRAB ("olivedrab", 0x6b8e23),
	DARKKHAKI ("darkkhaki", 0xbdb76b),
	KHAKI ("khaki", 0xf0e68c),
	PALEGOLDENROD ("palegoldenrod", 0xeee8aa),
	LIGHTGOLDENRODYELLOW ("lightgoldenrodyellow", 0xfafad2),
	LIGHTYELLOW ("lightyellow", 0xffffe0),
	YELLOW ("yellow", 0xffff00),
	GOLD ("gold", 0xffd700),
	LIGHTGOLDENROD ("lightgoldenrod", 0xeedd82),
	GOLDENROD ("goldenrod", 0xdaa520),
	DARKGOLDENROD ("darkgoldenrod", 0xb8860b),
	ROSYBROWN ("rosybrown", 0xbc8f8f),
	INDIANRED ("indianred", 0xcd5c5c),
	SADDLEBROWN ("saddlebrown", 0x8b4513),
	SIENNA ("sienna", 0xa0522d),
	PERU ("peru", 0xcd853f),
	BURLYWOOD ("burlywood", 0xdeb887),
	BEIGE ("beige", 0xf5f5dc),
	WHEAT ("wheat", 0xf5deb3),
	SANDYBROWN ("sandybrown", 0xf4a460),
	TAN ("tan", 0xd2b48c),
	CHOCOLATE ("chocolate", 0xd2691e),
	FIREBRICK ("firebrick", 0xb22222),
	BROWN ("brown", 0xa52a2a),
	DARKSALMON ("darksalmon", 0xe9967a),
	SALMON ("salmon", 0xfa8072),
	LIGHTSALMON ("lightsalmon", 0xffa07a),
	ORANGE ("orange", 0xffa500),
	DARKORANGE ("darkorange", 0xff8c00),
	CORAL ("coral", 0xff7f50),
	LIGHTCORAL ("lightcoral", 0xf08080),
	TOMATO ("tomato", 0xff6347),
	ORANGERED ("orangered", 0xff4500),
	RED ("red", 0xff0000),
	HOTPINK ("hotpink", 0xff69b4),
	DEEPPINK ("deeppink", 0xff1493),
	PINK ("pink", 0xffc0cb),
	LIGHTPINK ("lightpink", 0xffb6c1),
	PALEVIOLETRED ("palevioletred", 0xdb7093),
	MAROON ("maroon", 0xb03060),
	MEDIUMVIOLETRED ("mediumvioletred", 0xc71585),
	VIOLETRED ("violetred", 0xd02090),
	MAGENTA ("magenta", 0xff00ff),
	VIOLET ("violet", 0xee82ee),
	PLUM ("plum", 0xdda0dd),
	ORCHID ("orchid", 0xda70d6),
	MEDIUMORCHID ("mediumorchid", 0xba55d3),
	DARKORCHID ("darkorchid", 0x9932cc),
	DARKVIOLET ("darkviolet", 0x9400d3),
	BLUEVIOLET ("blueviolet", 0x8a2be2),
	PURPLE ("purple", 0xa020f0),
	MEDIUMPURPLE ("mediumpurple", 0x9370db),
	THISTLE ("thistle", 0xd8bfd8),
	SNOW1 ("snow1", 0xfffafa),
	SNOW2 ("snow2", 0xeee9e9),
	SNOW3 ("snow3", 0xcdc9c9),
	SNOW4 ("snow4", 0x8b8989),
	SEASHELL1 ("seashell1", 0xfff5ee),
	SEASHELL2 ("seashell2", 0xeee5de),
	SEASHELL3 ("seashell3", 0xcdc5bf),
	SEASHELL4 ("seashell4", 0x8b8682),
	ANTIQUEWHITE1 ("antiquewhite1", 0xffefdb),
	ANTIQUEWHITE2 ("antiquewhite2", 0xeedfcc),
	ANTIQUEWHITE3 ("antiquewhite3", 0xcdc0b0),
	ANTIQUEWHITE4 ("antiquewhite4", 0x8b8378),
	BISQUE1 ("bisque1", 0xffe4c4),
	BISQUE2 ("bisque2", 0xeed5b7),
	BISQUE3 ("bisque3", 0xcdb79e),
	BISQUE4 ("bisque4", 0x8b7d6b),
	PEACHPUFF1 ("peachpuff1", 0xffdab9),
	PEACHPUFF2 ("peachpuff2", 0xeecbad),
	PEACHPUFF3 ("peachpuff3", 0xcdaf95),
	PEACHPUFF4 ("peachpuff4", 0x8b7765),
	NAVAJOWHITE1 ("navajowhite1", 0xffdead),
	NAVAJOWHITE2 ("navajowhite2", 0xeecfa1),
	NAVAJOWHITE3 ("navajowhite3", 0xcdb38b),
	NAVAJOWHITE4 ("navajowhite4", 0x8b795e),
	LEMONCHIFFON1 ("lemonchiffon1", 0xfffacd),
	LEMONCHIFFON2 ("lemonchiffon2", 0xeee9bf),
	LEMONCHIFFON3 ("lemonchiffon3", 0xcdc9a5),
	LEMONCHIFFON4 ("lemonchiffon4", 0x8b8970),
	CORNSILK1 ("cornsilk1", 0xfff8dc),
	CORNSILK2 ("cornsilk2", 0xeee8cd),
	CORNSILK3 ("cornsilk3", 0xcdc8b1),
	CORNSILK4 ("cornsilk4", 0x8b8878),
	IVORY1 ("ivory1", 0xfffff0),
	IVORY2 ("ivory2", 0xeeeee0),
	IVORY3 ("ivory3", 0xcdcdc1),
	IVORY4 ("ivory4", 0x8b8b83),
	HONEYDEW1 ("honeydew1", 0xf0fff0),
	HONEYDEW2 ("honeydew2", 0xe0eee0),
	HONEYDEW3 ("honeydew3", 0xc1cdc1),
	HONEYDEW4 ("honeydew4", 0x838b83),
	LAVENDERBLUSH1 ("lavenderblush1", 0xfff0f5),
	LAVENDERBLUSH2 ("lavenderblush2", 0xeee0e5),
	LAVENDERBLUSH3 ("lavenderblush3", 0xcdc1c5),
	LAVENDERBLUSH4 ("lavenderblush4", 0x8b8386),
	MISTYROSE1 ("mistyrose1", 0xffe4e1),
	MISTYROSE2 ("mistyrose2", 0xeed5d2),
	MISTYROSE3 ("mistyrose3", 0xcdb7b5),
	MISTYROSE4 ("mistyrose4", 0x8b7d7b),
	AZURE1 ("azure1", 0xf0ffff),
	AZURE2 ("azure2", 0xe0eeee),
	AZURE3 ("azure3", 0xc1cdcd),
	AZURE4 ("azure4", 0x838b8b),
	SLATEBLUE1 ("slateblue1", 0x836fff),
	SLATEBLUE2 ("slateblue2", 0x7a67ee),
	SLATEBLUE3 ("slateblue3", 0x6959cd),
	SLATEBLUE4 ("slateblue4", 0x473c8b),
	ROYALBLUE1 ("royalblue1", 0x4876ff),
	ROYALBLUE2 ("royalblue2", 0x436eee),
	ROYALBLUE3 ("royalblue3", 0x3a5fcd),
	ROYALBLUE4 ("royalblue4", 0x27408b),
	BLUE1 ("blue1", 0x0000ff),
	BLUE2 ("blue2", 0x0000ee),
	BLUE3 ("blue3", 0x0000cd),
	BLUE4 ("blue4", 0x00008b),
	DODGERBLUE1 ("dodgerblue1", 0x1e90ff),
	DODGERBLUE2 ("dodgerblue2", 0x1c86ee),
	DODGERBLUE3 ("dodgerblue3", 0x1874cd),
	DODGERBLUE4 ("dodgerblue4", 0x104e8b),
	STEELBLUE1 ("steelblue1", 0x63b8ff),
	STEELBLUE2 ("steelblue2", 0x5cacee),
	STEELBLUE3 ("steelblue3", 0x4f94cd),
	STEELBLUE4 ("steelblue4", 0x36648b),
	DEEPSKYBLUE1 ("deepskyblue1", 0x00bfff),
	DEEPSKYBLUE2 ("deepskyblue2", 0x00b2ee),
	DEEPSKYBLUE3 ("deepskyblue3", 0x009acd),
	DEEPSKYBLUE4 ("deepskyblue4", 0x00688b),
	SKYBLUE1 ("skyblue1", 0x87ceff),
	SKYBLUE2 ("skyblue2", 0x7ec0ee),
	SKYBLUE3 ("skyblue3", 0x6ca6cd),
	SKYBLUE4 ("skyblue4", 0x4a708b),
	LIGHTSKYBLUE1 ("lightskyblue1", 0xb0e2ff),
	LIGHTSKYBLUE2 ("lightskyblue2", 0xa4d3ee),
	LIGHTSKYBLUE3 ("lightskyblue3", 0x8db6cd),
	LIGHTSKYBLUE4 ("lightskyblue4", 0x607b8b),
	SLATEGRAY1 ("slategray1", 0xc6e2ff),
	SLATEGRAY2 ("slategray2", 0xb9d3ee),
	SLATEGRAY3 ("slategray3", 0x9fb6cd),
	SLATEGRAY4 ("slategray4", 0x6c7b8b),
	LIGHTSTEELBLUE1 ("lightsteelblue1", 0xcae1ff),
	LIGHTSTEELBLUE2 ("lightsteelblue2", 0xbcd2ee),
	LIGHTSTEELBLUE3 ("lightsteelblue3", 0xa2b5cd),
	LIGHTSTEELBLUE4 ("lightsteelblue4", 0x6e7b8b),
	LIGHTBLUE1 ("lightblue1", 0xbfefff),
	LIGHTBLUE2 ("lightblue2", 0xb2dfee),
	LIGHTBLUE3 ("lightblue3", 0x9ac0cd),
	LIGHTBLUE4 ("lightblue4", 0x68838b),
	LIGHTCYAN1 ("lightcyan1", 0xe0ffff),
	LIGHTCYAN2 ("lightcyan2", 0xd1eeee),
	LIGHTCYAN3 ("lightcyan3", 0xb4cdcd),
	LIGHTCYAN4 ("lightcyan4", 0x7a8b8b),
	PALETURQUOISE1 ("paleturquoise1", 0xbbffff),
	PALETURQUOISE2 ("paleturquoise2", 0xaeeeee),
	PALETURQUOISE3 ("paleturquoise3", 0x96cdcd),
	PALETURQUOISE4 ("paleturquoise4", 0x668b8b),
	CADETBLUE1 ("cadetblue1", 0x98f5ff),
	CADETBLUE2 ("cadetblue2", 0x8ee5ee),
	CADETBLUE3 ("cadetblue3", 0x7ac5cd),
	CADETBLUE4 ("cadetblue4", 0x53868b),
	TURQUOISE1 ("turquoise1", 0x00f5ff),
	TURQUOISE2 ("turquoise2", 0x00e5ee),
	TURQUOISE3 ("turquoise3", 0x00c5cd),
	TURQUOISE4 ("turquoise4", 0x00868b),
	CYAN1 ("cyan1", 0x00ffff),
	CYAN2 ("cyan2", 0x00eeee),
	CYAN3 ("cyan3", 0x00cdcd),
	CYAN4 ("cyan4", 0x008b8b),
	DARKSLATEGRAY1 ("darkslategray1", 0x97ffff),
	DARKSLATEGRAY2 ("darkslategray2", 0x8deeee),
	DARKSLATEGRAY3 ("darkslategray3", 0x79cdcd),
	DARKSLATEGRAY4 ("darkslategray4", 0x528b8b),
	AQUAMARINE1 ("aquamarine1", 0x7fffd4),
	AQUAMARINE2 ("aquamarine2", 0x76eec6),
	AQUAMARINE3 ("aquamarine3", 0x66cdaa),
	AQUAMARINE4 ("aquamarine4", 0x458b74),
	DARKSEAGREEN1 ("darkseagreen1", 0xc1ffc1),
	DARKSEAGREEN2 ("darkseagreen2", 0xb4eeb4),
	DARKSEAGREEN3 ("darkseagreen3", 0x9bcd9b),
	DARKSEAGREEN4 ("darkseagreen4", 0x698b69),
	SEAGREEN1 ("seagreen1", 0x54ff9f),
	SEAGREEN2 ("seagreen2", 0x4eee94),
	SEAGREEN3 ("seagreen3", 0x43cd80),
	SEAGREEN4 ("seagreen4", 0x2e8b57),
	PALEGREEN1 ("palegreen1", 0x9aff9a),
	PALEGREEN2 ("palegreen2", 0x90ee90),
	PALEGREEN3 ("palegreen3", 0x7ccd7c),
	PALEGREEN4 ("palegreen4", 0x548b54),
	SPRINGGREEN1 ("springgreen1", 0x00ff7f),
	SPRINGGREEN2 ("springgreen2", 0x00ee76),
	SPRINGGREEN3 ("springgreen3", 0x00cd66),
	SPRINGGREEN4 ("springgreen4", 0x008b45),
	GREEN1 ("green1", 0x00ff00),
	GREEN2 ("green2", 0x00ee00),
	GREEN3 ("green3", 0x00cd00),
	GREEN4 ("green4", 0x008b00),
	CHARTREUSE1 ("chartreuse1", 0x7fff00),
	CHARTREUSE2 ("chartreuse2", 0x76ee00),
	CHARTREUSE3 ("chartreuse3", 0x66cd00),
	CHARTREUSE4 ("chartreuse4", 0x458b00),
	OLIVEDRAB1 ("olivedrab1", 0xc0ff3e),
	OLIVEDRAB2 ("olivedrab2", 0xb3ee3a),
	OLIVEDRAB3 ("olivedrab3", 0x9acd32),
	OLIVEDRAB4 ("olivedrab4", 0x698b22),
	DARKOLIVEGREEN1 ("darkolivegreen1", 0xcaff70),
	DARKOLIVEGREEN2 ("darkolivegreen2", 0xbcee68),
	DARKOLIVEGREEN3 ("darkolivegreen3", 0xa2cd5a),
	DARKOLIVEGREEN4 ("darkolivegreen4", 0x6e8b3d),
	KHAKI1 ("khaki1", 0xfff68f),
	KHAKI2 ("khaki2", 0xeee685),
	KHAKI3 ("khaki3", 0xcdc673),
	KHAKI4 ("khaki4", 0x8b864e),
	LIGHTGOLDENROD1 ("lightgoldenrod1", 0xffec8b),
	LIGHTGOLDENROD2 ("lightgoldenrod2", 0xeedc82),
	LIGHTGOLDENROD3 ("lightgoldenrod3", 0xcdbe70),
	LIGHTGOLDENROD4 ("lightgoldenrod4", 0x8b814c),
	LIGHTYELLOW1 ("lightyellow1", 0xffffe0),
	LIGHTYELLOW2 ("lightyellow2", 0xeeeed1),
	LIGHTYELLOW3 ("lightyellow3", 0xcdcdb4),
	LIGHTYELLOW4 ("lightyellow4", 0x8b8b7a),
	YELLOW1 ("yellow1", 0xffff00),
	YELLOW2 ("yellow2", 0xeeee00),
	YELLOW3 ("yellow3", 0xcdcd00),
	YELLOW4 ("yellow4", 0x8b8b00),
	GOLD1 ("gold1", 0xffd700),
	GOLD2 ("gold2", 0xeec900),
	GOLD3 ("gold3", 0xcdad00),
	GOLD4 ("gold4", 0x8b7500),
	GOLDENROD1 ("goldenrod1", 0xffc125),
	GOLDENROD2 ("goldenrod2", 0xeeb422),
	GOLDENROD3 ("goldenrod3", 0xcd9b1d),
	GOLDENROD4 ("goldenrod4", 0x8b6914),
	DARKGOLDENROD1 ("darkgoldenrod1", 0xffb90f),
	DARKGOLDENROD2 ("darkgoldenrod2", 0xeead0e),
	DARKGOLDENROD3 ("darkgoldenrod3", 0xcd950c),
	DARKGOLDENROD4 ("darkgoldenrod4", 0x8b6508),
	ROSYBROWN1 ("rosybrown1", 0xffc1c1),
	ROSYBROWN2 ("rosybrown2", 0xeeb4b4),
	ROSYBROWN3 ("rosybrown3", 0xcd9b9b),
	ROSYBROWN4 ("rosybrown4", 0x8b6969),
	INDIANRED1 ("indianred1", 0xff6a6a),
	INDIANRED2 ("indianred2", 0xee6363),
	INDIANRED3 ("indianred3", 0xcd5555),
	INDIANRED4 ("indianred4", 0x8b3a3a),
	SIENNA1 ("sienna1", 0xff8247),
	SIENNA2 ("sienna2", 0xee7942),
	SIENNA3 ("sienna3", 0xcd6839),
	SIENNA4 ("sienna4", 0x8b4726),
	BURLYWOOD1 ("burlywood1", 0xffd39b),
	BURLYWOOD2 ("burlywood2", 0xeec591),
	BURLYWOOD3 ("burlywood3", 0xcdaa7d),
	BURLYWOOD4 ("burlywood4", 0x8b7355),
	WHEAT1 ("wheat1", 0xffe7ba),
	WHEAT2 ("wheat2", 0xeed8ae),
	WHEAT3 ("wheat3", 0xcdba96),
	WHEAT4 ("wheat4", 0x8b7e66),
	TAN1 ("tan1", 0xffa54f),
	TAN2 ("tan2", 0xee9a49),
	TAN3 ("tan3", 0xcd853f),
	TAN4 ("tan4", 0x8b5a2b),
	CHOCOLATE1 ("chocolate1", 0xff7f24),
	CHOCOLATE2 ("chocolate2", 0xee7621),
	CHOCOLATE3 ("chocolate3", 0xcd661d),
	CHOCOLATE4 ("chocolate4", 0x8b4513),
	FIREBRICK1 ("firebrick1", 0xff3030),
	FIREBRICK2 ("firebrick2", 0xee2c2c),
	FIREBRICK3 ("firebrick3", 0xcd2626),
	FIREBRICK4 ("firebrick4", 0x8b1a1a),
	BROWN1 ("brown1", 0xff4040),
	BROWN2 ("brown2", 0xee3b3b),
	BROWN3 ("brown3", 0xcd3333),
	BROWN4 ("brown4", 0x8b2323),
	SALMON1 ("salmon1", 0xff8c69),
	SALMON2 ("salmon2", 0xee8262),
	SALMON3 ("salmon3", 0xcd7054),
	SALMON4 ("salmon4", 0x8b4c39),
	LIGHTSALMON1 ("lightsalmon1", 0xffa07a),
	LIGHTSALMON2 ("lightsalmon2", 0xee9572),
	LIGHTSALMON3 ("lightsalmon3", 0xcd8162),
	LIGHTSALMON4 ("lightsalmon4", 0x8b5742),
	ORANGE1 ("orange1", 0xffa500),
	ORANGE2 ("orange2", 0xee9a00),
	ORANGE3 ("orange3", 0xcd8500),
	ORANGE4 ("orange4", 0x8b5a00),
	DARKORANGE1 ("darkorange1", 0xff7f00),
	DARKORANGE2 ("darkorange2", 0xee7600),
	DARKORANGE3 ("darkorange3", 0xcd6600),
	DARKORANGE4 ("darkorange4", 0x8b4500),
	CORAL1 ("coral1", 0xff7256),
	CORAL2 ("coral2", 0xee6a50),
	CORAL3 ("coral3", 0xcd5b45),
	CORAL4 ("coral4", 0x8b3e2f),
	TOMATO1 ("tomato1", 0xff6347),
	TOMATO2 ("tomato2", 0xee5c42),
	TOMATO3 ("tomato3", 0xcd4f39),
	TOMATO4 ("tomato4", 0x8b3626),
	ORANGERED1 ("orangered1", 0xff4500),
	ORANGERED2 ("orangered2", 0xee4000),
	ORANGERED3 ("orangered3", 0xcd3700),
	ORANGERED4 ("orangered4", 0x8b2500),
	RED1 ("red1", 0xff0000),
	RED2 ("red2", 0xee0000),
	RED3 ("red3", 0xcd0000),
	RED4 ("red4", 0x8b0000),
	DEEPPINK1 ("deeppink1", 0xff1493),
	DEEPPINK2 ("deeppink2", 0xee1289),
	DEEPPINK3 ("deeppink3", 0xcd1076),
	DEEPPINK4 ("deeppink4", 0x8b0a50),
	HOTPINK1 ("hotpink1", 0xff6eb4),
	HOTPINK2 ("hotpink2", 0xee6aa7),
	HOTPINK3 ("hotpink3", 0xcd6090),
	HOTPINK4 ("hotpink4", 0x8b3a62),
	PINK1 ("pink1", 0xffb5c5),
	PINK2 ("pink2", 0xeea9b8),
	PINK3 ("pink3", 0xcd919e),
	PINK4 ("pink4", 0x8b636c),
	LIGHTPINK1 ("lightpink1", 0xffaeb9),
	LIGHTPINK2 ("lightpink2", 0xeea2ad),
	LIGHTPINK3 ("lightpink3", 0xcd8c95),
	LIGHTPINK4 ("lightpink4", 0x8b5f65),
	PALEVIOLETRED1 ("palevioletred1", 0xff82ab),
	PALEVIOLETRED2 ("palevioletred2", 0xee799f),
	PALEVIOLETRED3 ("palevioletred3", 0xcd6889),
	PALEVIOLETRED4 ("palevioletred4", 0x8b475d),
	MAROON1 ("maroon1", 0xff34b3),
	MAROON2 ("maroon2", 0xee30a7),
	MAROON3 ("maroon3", 0xcd2990),
	MAROON4 ("maroon4", 0x8b1c62),
	VIOLETRED1 ("violetred1", 0xff3e96),
	VIOLETRED2 ("violetred2", 0xee3a8c),
	VIOLETRED3 ("violetred3", 0xcd3278),
	VIOLETRED4 ("violetred4", 0x8b2252),
	MAGENTA1 ("magenta1", 0xff00ff),
	MAGENTA2 ("magenta2", 0xee00ee),
	MAGENTA3 ("magenta3", 0xcd00cd),
	MAGENTA4 ("magenta4", 0x8b008b),
	ORCHID1 ("orchid1", 0xff83fa),
	ORCHID2 ("orchid2", 0xee7ae9),
	ORCHID3 ("orchid3", 0xcd69c9),
	ORCHID4 ("orchid4", 0x8b4789),
	PLUM1 ("plum1", 0xffbbff),
	PLUM2 ("plum2", 0xeeaeee),
	PLUM3 ("plum3", 0xcd96cd),
	PLUM4 ("plum4", 0x8b668b),
	MEDIUMORCHID1 ("mediumorchid1", 0xe066ff),
	MEDIUMORCHID2 ("mediumorchid2", 0xd15fee),
	MEDIUMORCHID3 ("mediumorchid3", 0xb452cd),
	MEDIUMORCHID4 ("mediumorchid4", 0x7a378b),
	DARKORCHID1 ("darkorchid1", 0xbf3eff),
	DARKORCHID2 ("darkorchid2", 0xb23aee),
	DARKORCHID3 ("darkorchid3", 0x9a32cd),
	DARKORCHID4 ("darkorchid4", 0x68228b),
	PURPLE1 ("purple1", 0x9b30ff),
	PURPLE2 ("purple2", 0x912cee),
	PURPLE3 ("purple3", 0x7d26cd),
	PURPLE4 ("purple4", 0x551a8b),
	MEDIUMPURPLE1 ("mediumpurple1", 0xab82ff),
	MEDIUMPURPLE2 ("mediumpurple2", 0x9f79ee),
	MEDIUMPURPLE3 ("mediumpurple3", 0x8968cd),
	MEDIUMPURPLE4 ("mediumpurple4", 0x5d478b),
	THISTLE1 ("thistle1", 0xffe1ff),
	THISTLE2 ("thistle2", 0xeed2ee),
	THISTLE3 ("thistle3", 0xcdb5cd),
	THISTLE4 ("thistle4", 0x8b7b8b),
	GRAY0 ("gray0", 0x000000),
	GREY0 ("grey0", 0x000000),
	GRAY1 ("gray1", 0x030303),
	GREY1 ("grey1", 0x030303),
	GRAY2 ("gray2", 0x050505),
	GREY2 ("grey2", 0x050505),
	GRAY3 ("gray3", 0x080808),
	GREY3 ("grey3", 0x080808),
	GRAY4 ("gray4", 0x0a0a0a),
	GREY4 ("grey4", 0x0a0a0a),
	GRAY5 ("gray5", 0x0d0d0d),
	GREY5 ("grey5", 0x0d0d0d),
	GRAY6 ("gray6", 0x0f0f0f),
	GREY6 ("grey6", 0x0f0f0f),
	GRAY7 ("gray7", 0x121212),
	GREY7 ("grey7", 0x121212),
	GRAY8 ("gray8", 0x141414),
	GREY8 ("grey8", 0x141414),
	GRAY9 ("gray9", 0x171717),
	GREY9 ("grey9", 0x171717),
	GRAY10 ("gray10", 0x1a1a1a),
	GREY10 ("grey10", 0x1a1a1a),
	GRAY11 ("gray11", 0x1c1c1c),
	GREY11 ("grey11", 0x1c1c1c),
	GRAY12 ("gray12", 0x1f1f1f),
	GREY12 ("grey12", 0x1f1f1f),
	GRAY13 ("gray13", 0x212121),
	GREY13 ("grey13", 0x212121),
	GRAY14 ("gray14", 0x242424),
	GREY14 ("grey14", 0x242424),
	GRAY15 ("gray15", 0x262626),
	GREY15 ("grey15", 0x262626),
	GRAY16 ("gray16", 0x292929),
	GREY16 ("grey16", 0x292929),
	GRAY17 ("gray17", 0x2b2b2b),
	GREY17 ("grey17", 0x2b2b2b),
	GRAY18 ("gray18", 0x2e2e2e),
	GREY18 ("grey18", 0x2e2e2e),
	GRAY19 ("gray19", 0x303030),
	GREY19 ("grey19", 0x303030),
	GRAY20 ("gray20", 0x333333),
	GREY20 ("grey20", 0x333333),
	GRAY21 ("gray21", 0x363636),
	GREY21 ("grey21", 0x363636),
	GRAY22 ("gray22", 0x383838),
	GREY22 ("grey22", 0x383838),
	GRAY23 ("gray23", 0x3b3b3b),
	GREY23 ("grey23", 0x3b3b3b),
	GRAY24 ("gray24", 0x3d3d3d),
	GREY24 ("grey24", 0x3d3d3d),
	GRAY25 ("gray25", 0x404040),
	GREY25 ("grey25", 0x404040),
	GRAY26 ("gray26", 0x424242),
	GREY26 ("grey26", 0x424242),
	GRAY27 ("gray27", 0x454545),
	GREY27 ("grey27", 0x454545),
	GRAY28 ("gray28", 0x474747),
	GREY28 ("grey28", 0x474747),
	GRAY29 ("gray29", 0x4a4a4a),
	GREY29 ("grey29", 0x4a4a4a),
	GRAY30 ("gray30", 0x4d4d4d),
	GREY30 ("grey30", 0x4d4d4d),
	GRAY31 ("gray31", 0x4f4f4f),
	GREY31 ("grey31", 0x4f4f4f),
	GRAY32 ("gray32", 0x525252),
	GREY32 ("grey32", 0x525252),
	GRAY33 ("gray33", 0x545454),
	GREY33 ("grey33", 0x545454),
	GRAY34 ("gray34", 0x575757),
	GREY34 ("grey34", 0x575757),
	GRAY35 ("gray35", 0x595959),
	GREY35 ("grey35", 0x595959),
	GRAY36 ("gray36", 0x5c5c5c),
	GREY36 ("grey36", 0x5c5c5c),
	GRAY37 ("gray37", 0x5e5e5e),
	GREY37 ("grey37", 0x5e5e5e),
	GRAY38 ("gray38", 0x616161),
	GREY38 ("grey38", 0x616161),
	GRAY39 ("gray39", 0x636363),
	GREY39 ("grey39", 0x636363),
	GRAY40 ("gray40", 0x666666),
	GREY40 ("grey40", 0x666666),
	GRAY41 ("gray41", 0x696969),
	GREY41 ("grey41", 0x696969),
	GRAY42 ("gray42", 0x6b6b6b),
	GREY42 ("grey42", 0x6b6b6b),
	GRAY43 ("gray43", 0x6e6e6e),
	GREY43 ("grey43", 0x6e6e6e),
	GRAY44 ("gray44", 0x707070),
	GREY44 ("grey44", 0x707070),
	GRAY45 ("gray45", 0x737373),
	GREY45 ("grey45", 0x737373),
	GRAY46 ("gray46", 0x757575),
	GREY46 ("grey46", 0x757575),
	GRAY47 ("gray47", 0x787878),
	GREY47 ("grey47", 0x787878),
	GRAY48 ("gray48", 0x7a7a7a),
	GREY48 ("grey48", 0x7a7a7a),
	GRAY49 ("gray49", 0x7d7d7d),
	GREY49 ("grey49", 0x7d7d7d),
	GRAY50 ("gray50", 0x7f7f7f),
	GREY50 ("grey50", 0x7f7f7f),
	GRAY51 ("gray51", 0x828282),
	GREY51 ("grey51", 0x828282),
	GRAY52 ("gray52", 0x858585),
	GREY52 ("grey52", 0x858585),
	GRAY53 ("gray53", 0x878787),
	GREY53 ("grey53", 0x878787),
	GRAY54 ("gray54", 0x8a8a8a),
	GREY54 ("grey54", 0x8a8a8a),
	GRAY55 ("gray55", 0x8c8c8c),
	GREY55 ("grey55", 0x8c8c8c),
	GRAY56 ("gray56", 0x8f8f8f),
	GREY56 ("grey56", 0x8f8f8f),
	GRAY57 ("gray57", 0x919191),
	GREY57 ("grey57", 0x919191),
	GRAY58 ("gray58", 0x949494),
	GREY58 ("grey58", 0x949494),
	GRAY59 ("gray59", 0x969696),
	GREY59 ("grey59", 0x969696),
	GRAY60 ("gray60", 0x999999),
	GREY60 ("grey60", 0x999999),
	GRAY61 ("gray61", 0x9c9c9c),
	GREY61 ("grey61", 0x9c9c9c),
	GRAY62 ("gray62", 0x9e9e9e),
	GREY62 ("grey62", 0x9e9e9e),
	GRAY63 ("gray63", 0xa1a1a1),
	GREY63 ("grey63", 0xa1a1a1),
	GRAY64 ("gray64", 0xa3a3a3),
	GREY64 ("grey64", 0xa3a3a3),
	GRAY65 ("gray65", 0xa6a6a6),
	GREY65 ("grey65", 0xa6a6a6),
	GRAY66 ("gray66", 0xa8a8a8),
	GREY66 ("grey66", 0xa8a8a8),
	GRAY67 ("gray67", 0xababab),
	GREY67 ("grey67", 0xababab),
	GRAY68 ("gray68", 0xadadad),
	GREY68 ("grey68", 0xadadad),
	GRAY69 ("gray69", 0xb0b0b0),
	GREY69 ("grey69", 0xb0b0b0),
	GRAY70 ("gray70", 0xb3b3b3),
	GREY70 ("grey70", 0xb3b3b3),
	GRAY71 ("gray71", 0xb5b5b5),
	GREY71 ("grey71", 0xb5b5b5),
	GRAY72 ("gray72", 0xb8b8b8),
	GREY72 ("grey72", 0xb8b8b8),
	GRAY73 ("gray73", 0xbababa),
	GREY73 ("grey73", 0xbababa),
	GRAY74 ("gray74", 0xbdbdbd),
	GREY74 ("grey74", 0xbdbdbd),
	GRAY75 ("gray75", 0xbfbfbf),
	GREY75 ("grey75", 0xbfbfbf),
	GRAY76 ("gray76", 0xc2c2c2),
	GREY76 ("grey76", 0xc2c2c2),
	GRAY77 ("gray77", 0xc4c4c4),
	GREY77 ("grey77", 0xc4c4c4),
	GRAY78 ("gray78", 0xc7c7c7),
	GREY78 ("grey78", 0xc7c7c7),
	GRAY79 ("gray79", 0xc9c9c9),
	GREY79 ("grey79", 0xc9c9c9),
	GRAY80 ("gray80", 0xcccccc),
	GREY80 ("grey80", 0xcccccc),
	GRAY81 ("gray81", 0xcfcfcf),
	GREY81 ("grey81", 0xcfcfcf),
	GRAY82 ("gray82", 0xd1d1d1),
	GREY82 ("grey82", 0xd1d1d1),
	GRAY83 ("gray83", 0xd4d4d4),
	GREY83 ("grey83", 0xd4d4d4),
	GRAY84 ("gray84", 0xd6d6d6),
	GREY84 ("grey84", 0xd6d6d6),
	GRAY85 ("gray85", 0xd9d9d9),
	GREY85 ("grey85", 0xd9d9d9),
	GRAY86 ("gray86", 0xdbdbdb),
	GREY86 ("grey86", 0xdbdbdb),
	GRAY87 ("gray87", 0xdedede),
	GREY87 ("grey87", 0xdedede),
	GRAY88 ("gray88", 0xe0e0e0),
	GREY88 ("grey88", 0xe0e0e0),
	GRAY89 ("gray89", 0xe3e3e3),
	GREY89 ("grey89", 0xe3e3e3),
	GRAY90 ("gray90", 0xe5e5e5),
	GREY90 ("grey90", 0xe5e5e5),
	GRAY91 ("gray91", 0xe8e8e8),
	GREY91 ("grey91", 0xe8e8e8),
	GRAY92 ("gray92", 0xebebeb),
	GREY92 ("grey92", 0xebebeb),
	GRAY93 ("gray93", 0xededed),
	GREY93 ("grey93", 0xededed),
	GRAY94 ("gray94", 0xf0f0f0),
	GREY94 ("grey94", 0xf0f0f0),
	GRAY95 ("gray95", 0xf2f2f2),
	GREY95 ("grey95", 0xf2f2f2),
	GRAY96 ("gray96", 0xf5f5f5),
	GREY96 ("grey96", 0xf5f5f5),
	GRAY97 ("gray97", 0xf7f7f7),
	GREY97 ("grey97", 0xf7f7f7),
	GRAY98 ("gray98", 0xfafafa),
	GREY98 ("grey98", 0xfafafa),
	GRAY99 ("gray99", 0xfcfcfc),
	GREY99 ("grey99", 0xfcfcfc),
	GRAY100 ("gray100", 0xffffff),
	GREY100 ("grey100", 0xffffff),
	DARKGREY ("darkgrey", 0xa9a9a9),
	DARKGRAY ("darkgray", 0xa9a9a9),
	DARKBLUE ("darkblue", 0x00008b),
	DARKCYAN ("darkcyan", 0x008b8b),
	DARKMAGENTA ("darkmagenta", 0x8b008b),
	DARKRED ("darkred", 0x8b0000),
	LIGHTGREEN ("lightgreen", 0x90ee90);

	private String label;
	private int color;
	private static Map<String, ColorKeyword>cMap;

	ColorKeyword(String label, int color) { 
		this.label = label; 
		this.color = color; 
		addKeyword(this);
	}

	public String getLabel() {
		return label;
	}

	public int getColorCode() {
		return color;
	}

	@Override
	public String toString() {
		return label;
	}
	
	private void addKeyword(ColorKeyword col) {
		if (cMap == null) cMap = new HashMap<String,ColorKeyword>();
		cMap.put(col.getLabel(), col);
	}

	public static Color getColor(String label) {
		if (!cMap.containsKey(label))
			return null;
		return new Color(cMap.get(label).getColorCode());
	}
}
